#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import time
import csv
import datetime

ROOT_DIR = '/pool2/archive/public/'

DOM_EXT = ['.ru-', '.org-', '.su-', '.com-', '.net-', '.xn--p1ai-', '.io-']

DIRNAMES = ['mirrors', 'webarchive', 'webcollect', 'webcollect2017', 'webcollect2018', 'webcollect2019',
            'webcollect2020', 'webtimedcoll', 'wikicollect', 'webcollect2021', 'webcollect2022', 'webcollect2023', 'nkr']

PUBLIC_PREFIX = 'https://cdn.ruarxive.org/public'


DIR_TYPES_MAP = {
    'nkr' : 'collections',
    'mirrors': 'simple',
    'datacollect' : 'collections',
    'dataportals' : 'collections',
    'webarchive': 'collections',
    'webcollect': 'collections',
    'webcollect2017': 'collections',
    'webcollect2018': 'collections',
    'webcollect2019': 'collections',
    'webcollect2020': 'collections',
    'webcollect2021': 'collections',
    'webcollect2022': 'collections',
    'webcollect2023': 'collections',
    'webtimedcoll': 'timed',
    'wikicollect': 'simple'
}


def simple_list_files(dirname, coll_name):
    total = 0
    records = []
    filemap = {}
    listfile = os.path.join(dirname, 'list.txt')
    if os.path.exists(listfile):
        f = open(listfile, 'r', encoding='utf8')
        rows = csv.DictReader(f, delimiter=',')
        for r in rows:
            filemap[r['filename']] = r['website']
        f.close()
    files = os.listdir(dirname)
    for afile in files:
        suffix = afile.rsplit('.', 1)[-1]
        if suffix not in ['gz', '7z', 'xz', 'zip']: continue
        if len(filemap.keys()) > 0 and afile.rsplit('.', 1)[0] in filemap.keys():
            domain = filemap[afile.rsplit('.', 1)[0]]
        else:
            domain = afile.rsplit('.', 1)[0]
        filepath = os.path.join(dirname, afile)
        mtime = os.path.getmtime(filepath)
        last_modified_date = datetime.datetime.fromtimestamp(mtime)
        fsize = os.path.getsize(filepath)
        total += fsize
        public_url = PUBLIC_PREFIX + '/' + filepath
        record = [domain, coll_name, domain, afile, str(fsize), str(last_modified_date), public_url]
        records.append(record)
    return records, total


def process_dir(source_dir, coll_name):
    total = 0
    records = []
    sites = []
    fullp = os.path.join(source_dir, coll_name)
    dirs = os.listdir(fullp)
    for d in dirs:
        if not os.path.isdir(os.path.join(fullp, d)): continue
        sites.append(d)
    domain = ''
    for site in sites:
        if site.find('-') == -1:
            domain = site
        else:
            for ext in DOM_EXT:
                if site.find(ext) == -1:
                    continue
                domain = site.split(ext, 1)[0] + ext[0:-1]
                break
        sitep = os.path.join(fullp, site)
        files = os.listdir(sitep)
        for afile in files:
            filepath = os.path.join(sitep, afile)
            mtime = os.path.getmtime(filepath)
            last_modified_date = datetime.datetime.fromtimestamp(mtime)
            fsize = os.path.getsize(filepath)
            total += fsize
            public_url = PUBLIC_PREFIX + '/' + filepath
            record = [site, coll_name, domain, afile, str(fsize), str(last_modified_date), public_url]
            records.append(record)
    return records, total


def filereport(groups=DIR_TYPES_MAP):
    keys = ['id', 'collection', 'domain', 'filename', 'filesize', 'date_modified', 'public_url']
    print(','.join(keys))
    for dirname, dirtype in groups.items():
        collections = []
        if dirtype == 'collections':
            source_dir = os.path.join(ROOT_DIR, dirname)
            dirs = os.listdir(source_dir)
            for d in dirs:
                if not os.path.isdir(os.path.join(source_dir, d)):
                    continue
                collections.append(d)
            for c in collections:
                records, c_total = process_dir(dirname, c)
                for r in records:
                    print(','.join(r))
        elif dirtype == 'simple':
            source_dir = os.path.join(ROOT_DIR, dirname)
            records, c_total = simple_list_files(dirname, dirname)
            for r in records:
                print(','.join(r))
        elif dirtype == 'timed':
            source_dir = os.path.join(ROOT_DIR, dirname)
            dirs = os.listdir(source_dir)
            for d in dirs:
                if not os.path.isdir(os.path.join(source_dir, d)):
                    continue
                collections.append(d)
                crawls = os.listdir(os.path.join(dirname, d))
                for cr in crawls:
                    dirs = os.listdir(os.path.join(dirname, d, cr))
                    for dir in dirs:
                        records, c_total = process_dir(os.path.join(dirname, d), cr)
                        for r in records:
                            print(','.join(r))
    pass


def collreport():
    total = 0
    collections = []
    colls = []
    keys = ['collection', 'num_sites', 'storage_size']
    print(','.join(keys))
    dirs = os.listdir(SOURCEDIR)
    for d in dirs:
        if not os.path.isdir(os.path.join(SOURCEDIR, d)):
            continue
        collections.append(d)
    for c in collections:
        item = {}
        sites = []
        fullp = os.path.join(SOURCEDIR, c)
        dirs = os.listdir(fullp)
        for d in dirs:
            if not os.path.isdir(os.path.join(fullp, d)): continue
            sites.append(d)
        n_sites = len(sites)
        storage_size = 0
        for site in sites:
            sitep = os.path.join(fullp, site)
            files = os.listdir(sitep)
            for afile in files:
                filepath = os.path.join(sitep, afile)
                mtime = os.path.getmtime(filepath)
                last_modified_date = datetime.datetime.fromtimestamp(mtime)
                fsize = os.path.getsize(filepath)
                total += fsize
                storage_size += fsize
                url = URL_PATH + '%s/%s/%s' % (c, site, afile)
                record = [c, site, afile, str(fsize), str(last_modified_date), url]
        item = [c, str(n_sites), str(storage_size)]
        print(','.join(item))
        colls.append(item)


if __name__ == "__main__":
    filereport(groups=DIR_TYPES_MAP)
#    collreport()
